﻿Type=Activity
Version=5.2
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
#Region  Activity Attributes 
	#FullScreen: False
	#IncludeTitle: True
#End Region

Sub Process_Globals
	'These global variables will be declared once when the application starts.
	'These variables can be accessed from all modules.

End Sub

Sub Globals
	'These global variables will be redeclared each time the activity is created.
	'These variables can only be accessed from this module.

	Private GameMachineCostEditText As EditText
	Private GameMachineIDEditText As EditText
	Private GameMachineMakerEditText As EditText
	Private GameMachineMostPowerfulCheckBox As CheckBox
	Private GameMachineNameEditText As EditText
	Private GameMachinesListView As ListView
	Private GamesListView As ListView
	Private LoadGameMachinesButton As Button

	Type GameMachine (GameMachineID As Int,	MachineName As String, MachineMaker As String, MachineCost As Double, MostPowerful As Boolean)
	Type Game (GameID As Int, GameMachineID As Int, GameName As String, GameDev As String, PlatformExclusive As Boolean, EGRating As Int)
	
	'these variables are the main data structure ones, storing the data and providing access to the currently selected item
	Private GamesMachinesList As List
	Private GamesList As List
	Private SelectedGamesMachine As GameMachine
	Private SelectedGame As Game
	
	
	Private AddMachineButton As Button
	Private DeleteMachineButton As Button
	Private UpdateMachineButton As Button

	Private CopyDefaultFilesButton As Button
	Private AddGameButton As Button
	Private DeleteGameButton As Button
	Private GameDevEditText As EditText
	Private GameMultiplatformCheckBox As CheckBox
	Private GameNameEditText As EditText
	Private UpdateGameButton As Button
	Private ManualCSVButton As Button	
	Private FullyManualImportBtn As Button
	
	
	Private ManualCSVExportButton As Button
	Private GamesSortSpinner As Spinner
	Private GamesFilterSpinner As Spinner
	Private GamesFilterEditText As EditText
	Private GameFilterOperatorSpinner As Spinner
	Private GameEGRatingText As EditText
End Sub

Sub Activity_Create(FirstTime As Boolean)
	'Do not forget to load the layout file created with the visual designer. For example:
	Activity.LoadLayout("RelationalDataScreen")	
	
	Activity.AddMenuItem3("Return to Main Screen","MainScreenMenu",LoadBitmap(File.DirAssets,"MBison.gif"),True)
	
	'initialise data structures
	GamesMachinesList.Initialize
	GamesList.Initialize
	
	'set up control spinners
	PopulateControlSpinners
	
End Sub

Sub Activity_Resume

End Sub

Sub Activity_Pause (UserClosed As Boolean)

End Sub

Sub MainScreenMenu_Click	
	StartActivity(Main)	
End Sub

Sub LoadGameMachinesButton_Click
	'This sub will load in the necessary files and place the data on the screen
	
	'first need to check if the system files exist in the App's file folder and if not copy them over from the defaults
	'NOTE: instead of copying them over could just create empty files 
	If (File.Exists(File.DirDefaultExternal,"GameMachines.csv") = False) Then
		'getting here means need to copy over the file
		File.Copy(File.DirAssets,"GameMachines.csv",File.DirDefaultExternal,"GameMachines.csv")	
	End If
	
	If (File.Exists(File.DirDefaultExternal,"Games.csv") = False) Then
		'getting here means need to copy over the file
		File.Copy(File.DirAssets,"Games.csv",File.DirDefaultExternal,"Games.csv")	
	End If	
	
	'this loads the files into the data structures 
	LoadInFiles
	
	'this updates the Machines screen controls with the data from the data structures
	PopulateMachinesListView(True)	
End Sub

Sub LoadInFiles
	'This sub will read in the default files from the File Assets folder into the list objects
	
	Dim SU As StringUtils
	Dim ImportDataList As List 'this will hold all the data read in from the file
	Dim ImportDataLine() As String 'this will hold one line's data
	
	
	'FIRST: do the Games Machines
	
	GamesMachinesList.Initialize 'initialize the list data structure to hold the films
		
	'first need to read the CSV file of films in a list object
	ImportDataList = SU.LoadCSV(File.DirDefaultExternal,"GameMachines.csv",",")
	
	'now need to read through each item in the input list and convert it into a FilmRecord object
	For i=0 To ImportDataList.Size-1
	
		'need to declare and initialize at this point the object going to read in
		Dim ReadInMachine As GameMachine
		ReadInMachine.Initialize
	
		'first need to get the array of strings held in the current list item
		ImportDataLine = ImportDataList.Get(i)
		
		'now because we know the data structure of the file we are reading in we can refer to items in the data string array directly
		ReadInMachine.GameMachineID = ImportDataLine(0) 
		ReadInMachine.MachineName = ImportDataLine(1) 
		ReadInMachine.MachineMaker = ImportDataLine(2)
		ReadInMachine.MachineCost = ImportDataLine(3)
		
		'for the OnNetflix field as it's boolean we need to read the string and act accordingly
		If (ImportDataLine(4) = "TRUE") Then
			ReadInMachine.MostPowerful = True
		Else
			ReadInMachine.MostPowerful = False
		End If
		
		'Now can add the ReadInFilm object to the FilmList
		GamesMachinesList.Add(ReadInMachine)		
	Next
	

	'SECOND: do the Games
	GamesList.Initialize 'initialize the list data structure to hold the films
	ImportDataList.Clear
		
	'first need to read the CSV file of films in a list object
	ImportDataList = SU.LoadCSV(File.DirDefaultExternal,"Games.csv",",")
	
	'now need to read through each item in the input list and convert it into a FilmRecord object
	For i=0 To ImportDataList.Size-1
	
		'need to declare and initialize at this point the object going to read in
		Dim ReadInGame As Game
		ReadInGame.Initialize
	
		'first need to get the array of strings held in the current list item
		ImportDataLine = ImportDataList.Get(i)
		
		'now because we know the data structure of the file we are reading in we can refer to items in the data string array directly
		ReadInGame.GameID = ImportDataLine(0) 
		ReadInGame.GameMachineID = ImportDataLine(1) 
		ReadInGame.GameName = ImportDataLine(2)
		ReadInGame.GameDev = ImportDataLine(3)
		ReadInGame.EGRating = ImportDataLine(5)
		
		'for the OnNetflix field as it's boolean we need to read the string and act accordingly
		If (ImportDataLine(4) = "TRUE") Then
			ReadInGame.PlatformExclusive = True
		Else
			ReadInGame.PlatformExclusive = False
		End If
		
		'Now can add the ReadInFilm object to the FilmList
		GamesList.Add(ReadInGame)		
	Next
	
End Sub

Sub UpdateDataFiles
	'this sub routine will update the data files in the device's storage with whatever is in the current data structures
	'NOTE: this overwrites whatever is currently in the files it does NOT append new data to the end or edit existing data
	
	Dim WriteList As List
	Dim WriteMachine As GameMachine
	Dim WriteGame As Game
	
	Dim SU As StringUtils 'object that provides interface for writing to files
	
	'FIRST write out the GamesMachine file
	WriteList.Initialize
	
	For i=0 To GamesMachinesList.Size-1
		WriteMachine = GamesMachinesList.Get(i)
		
		'IMPORTANT NOTE: it is crucial the write format of the file is what will be expected by the read sub
		WriteList.Add(Array As String(WriteMachine.GameMachineID, WriteMachine.MachineName, WriteMachine.MachineMaker, WriteMachine.MachineCost, WriteMachine.MostPowerful))
	Next
	
	'error checking: only write the file if there is data to write
	If (WriteList.Size > 0) Then	
		'now write the list to the file
		SU.SaveCSV(File.DirDefaultExternal, "GameMachines.csv", ",", WriteList)	
	End If 
	
	
	'SECOND write out the Games file
	WriteList.Initialize
	
	For i=0 To GamesList.Size-1
		WriteGame = GamesList.Get(i)
		
		'IMPORTANT NOTE: it is crucial the write format of the file is what will be expected by the read sub
		WriteList.Add(Array As String(WriteGame.GameID, WriteGame.GameMachineID, WriteGame.GameName, WriteGame.GameDev, WriteGame.PlatformExclusive, WriteGame.EGRating))
		
	Next

	'error checking: only write the file if there is data to write
	If (WriteList.Size > 0) Then	
		'now write the list to the file
		SU.SaveCSV(File.DirDefaultExternal, "Games.csv", ",", WriteList)	
	End If

End Sub

Sub PopulateMachinesListView(EmptyGamesListView As Boolean)
	'this sub will populate the Games machines list view with all the machines
	
	Dim DisplayGameMachine As GameMachine 'this will be used to add new items to the List View
	
	GameMachinesListView.Clear
	
	For i=0 To GamesMachinesList.Size-1
		'get hold of the current record in the list		
		DisplayGameMachine = GamesMachinesList.Get(i)
		
		'now add the machine to the list view, using data accessed from it and linking it to that object
		GameMachinesListView.AddTwoLines2(DisplayGameMachine.MachineName,DisplayGameMachine.MachineMaker,DisplayGameMachine)	
	Next
	
	'depending on the passed in parameter clear the games list to make it clear that no machine is currently selected
	If (EmptyGamesListView) Then
		GamesListView.Clear
	End If
	
	'also clear any selected items
	'SelectedGamesMachine = Null
	'SelectedGame = Null
	
	
End Sub

Sub PopulateGamesListView(ParentKey As Int)
	'this sub will populate the games list view with all the games from the list THAT have the corresponding parent key
	Dim DisplayGame As Game
	Dim GameDisplayString As String
	Dim PassesFilter As Boolean
	Dim FilterText As String
	Dim SF As StringFunctions
	
	
	GamesListView.Clear	
	
	'check if a Sort has been selected and if so apply it
	If (GamesSortSpinner.SelectedItem <> "Default") Then
		
		If (GamesSortSpinner.SelectedItem = "Game Name") Then
			GamesList.SortType("GameName",True)
		End If
	
		If (GamesSortSpinner.SelectedItem = "Game Dev") Then
			GamesList.SortType("GameDev",True)
		End If

		If (GamesSortSpinner.SelectedItem = "Platform Exclusive") Then
			GamesList.SortType("PlatformExclusive",True)
		End If
	
		If (GamesSortSpinner.SelectedItem = "EG Rating") Then
			GamesList.SortType("EGRating",False) 'note: sort by descending, e.g. highest scoring games first
		End If
	
	End If	
	
	For i=0 To GamesList.Size-1
	
		'get hold of the current record, then determine if it needs processing
		DisplayGame = GamesList.Get(i)
		If (DisplayGame.GameMachineID = ParentKey) Then
			'getting here means it is a game, may add it to the list - depending on filter
			
			'only filter if a field is selected AND there is filter text given
			If ((GamesFilterSpinner.SelectedItem = "No Filter") Or (GamesFilterEditText.Text = "")) Then				
				'no filter is applied so can just add normally				
				GameDisplayString = DisplayGame.GameName & ", " & DisplayGame.GameDev & ", " & DisplayGame.EGRating & "/10"
				
				'depending on the multiplaform status of the game change the second line
				If (DisplayGame.PlatformExclusive) Then
					GamesListView.AddTwoLines2(GameDisplayString, "Exclusive to this Machine",DisplayGame)
				Else
					GamesListView.AddTwoLines2(GameDisplayString, "Multi-platform",DisplayGame)
				End If		
			Else
				'filter is applied so need to check each possible item
				'first presume no filter will be applied and then turn it on depending on check
				PassesFilter = True
				
				'get the filter text, stripping away any trailing blank spaces
				FilterText = SF.Rtrim(GamesFilterEditText.Text)
				
				If (GamesFilterSpinner.SelectedItem = "Game Name") Then
					If (DisplayGame.GameName <> FilterText) Then
						PassesFilter = False
					End If					
				End If
				
				'Log("Object Game Dev: " & 				
				
				If (GamesFilterSpinner.SelectedItem = "Game Dev") Then
					If (DisplayGame.GameDev <> FilterText) Then
						PassesFilter = False
					End If					
				End If
				
				If (GamesFilterSpinner.SelectedItem = "Platform Exclusive") Then
					'have to do two tests here, both for Yes and No
					If ((DisplayGame.PlatformExclusive = True) And (FilterText <> "Y")) Then
						PassesFilter = False						
					End If
				
					If ((DisplayGame.PlatformExclusive = False) And (FilterText <> "N")) Then
						PassesFilter = False						
					End If			
				End If
			
			
				If (GamesFilterSpinner.SelectedItem = "EG Rating") Then
					'filtering on a number field so check user input is a number
					If (IsNumber(FilterText)) Then
						'as this is number field need to also use the operator	
						If (GameFilterOperatorSpinner.SelectedItem = "=") Then
							If (DisplayGame.EGRating <> FilterText) Then
								PassesFilter = False
							End If					
						End If
					
						If (GameFilterOperatorSpinner.SelectedItem = ">") Then
							If ((DisplayGame.EGRating < FilterText) OR (DisplayGame.EGRating = FilterText)) Then
								PassesFilter = False
							End If					
						End If
					
						If (GameFilterOperatorSpinner.SelectedItem = "<") Then
							If ((DisplayGame.EGRating > FilterText) OR (DisplayGame.EGRating = FilterText)) Then
								PassesFilter = False
							End If					
						End If				
					Else
						'user wanted to filter on a number field but didn't enter a number so nothing will pass the filter
						PassesFilter = False
					End If
				End If
				
				'if have reached here and passes filter is still true then it means the game and be added to the list view
				If (PassesFilter) Then
					'no filter is applied so can just add normally				
					GameDisplayString = DisplayGame.GameName & ", " & DisplayGame.GameDev & ", " & DisplayGame.EGRating & "/10"
					
					'depending on the multiplaform status of the game change the second line
					If (DisplayGame.PlatformExclusive) Then
						GamesListView.AddTwoLines2(GameDisplayString, "Exclusive to this Machine",DisplayGame)
					Else
						GamesListView.AddTwoLines2(GameDisplayString, "Multi-platform",DisplayGame)
					End If				
				End If
			
			End If
		End If	
	Next

End Sub




Sub GamesListView_ItemClick (Position As Int, Value As Object)
	'select the current game and fill the edit controls
	SelectedGame = Value	
	
	GameNameEditText.Text = SelectedGame.GameName
	GameDevEditText.Text = SelectedGame.GameDev
	GameMultiplatformCheckBox.Checked = SelectedGame.PlatformExclusive
	GameEGRatingText.Text = SelectedGame.EGRating
	
	'enable the buttons
	AddGameButton.Enabled = True
	UpdateGameButton.Enabled = True
	DeleteGameButton.Enabled = True	
	
End Sub

Sub GameMachinesListView_ItemClick (Position As Int, Value As Object)
	
	'get hold of the selected machine
	Dim ClickedMachine As GameMachine
	
	ClickedMachine = Value
	
	'display its data in the edit boxes
	GameMachineIDEditText.Text = ClickedMachine.GameMachineID
	GameMachineNameEditText.Text = ClickedMachine.MachineName	
	GameMachineMakerEditText.Text = ClickedMachine.MachineMaker
	GameMachineCostEditText.Text = ClickedMachine.MachineCost
	GameMachineMostPowerfulCheckBox.Checked = ClickedMachine.MostPowerful
	
	'update the global reference to the selected machine
	SelectedGamesMachine = Value
	
	'clear the selected game and its controls
	SelectedGame = Null
	GameNameEditText.Text = ""
	GameDevEditText.Text = ""
	GameMultiplatformCheckBox.Checked = False
	AddGameButton.Enabled = True 'allow games to be added immediately a machine is selected 
	UpdateGameButton.Enabled = False 
	DeleteGameButton.Enabled = False 
	
	'enable the various record options now an item is selected
	UpdateMachineButton.Enabled = True
	DeleteMachineButton.Enabled = True
	
	'have it's games displayed in the list view
	PopulateGamesListView(ClickedMachine.GameMachineID)
	
End Sub

Sub GetNewGameMachineID() As Int
	'this function will return the next ID number that should be used for a new GamesMachine record
	'OBVIOUSLY it must be called immediately before a record is inserted AND IS NOT RELIABLE for multi user scenarios (which is currently far beyond the scope of this app)
	
	Dim ReturnID As Int
	ReturnID = 0
	
	Dim CurrentMachine As GameMachine
	
	'iterate through all the items in the list until find the largest key
	For i=0 To GamesMachinesList.Size-1
		CurrentMachine = GamesMachinesList.Get(i)
		
		If (CurrentMachine.GameMachineID > ReturnID) Then
			ReturnID = CurrentMachine.GameMachineID
		End If	
	
	Next
	
	'the new key will now be the current largest ID so just add 1 to it
	ReturnID = ReturnID + 1
	
	Return ReturnID

End Sub

Sub GetNewGameID() As Int
	'this function will return the next ID number that should be used for a new Game record
	'OBVIOUSLY it must be called immediately before a record is inserted AND IS NOT RELIABLE for multi user scenarios (which is currently far beyond the scope of this app)
	
	Dim ReturnID As Int
	ReturnID = 0
	
	Dim CurrentGame As Game
	
	'iterate through all the items in the list until find the largest key
	For i=0 To GamesList.Size-1
		CurrentGame = GamesList.Get(i)
		
		If (CurrentGame.GameID > ReturnID) Then
			ReturnID = CurrentGame.GameID
		End If	
	
	Next
	
	'the new key will now be the current largest ID so just add 1 to it
	ReturnID = ReturnID + 1
	
	Return ReturnID

End Sub


Sub UpdateMachineButton_Click
	'this sub will update the selected machine with the data currently in the edit controls
	
	If (SelectedGamesMachine = Null) Then
		Msgbox("Cannot update - No Machine is selected, in fact this should never be possible...","Update Error")
	End If
	
	'update the data in the object
	SelectedGamesMachine.MachineCost = GameMachineCostEditText.Text
	SelectedGamesMachine.MachineMaker = GameMachineMakerEditText.Text
	SelectedGamesMachine.MachineName = GameMachineNameEditText.Text
	SelectedGamesMachine.MostPowerful = GameMachineMostPowerfulCheckBox.Checked

	'have the on screen controls updated
	PopulateMachinesListView(False) 'don't want the games list view to be changed
		
	'have the data files reflect the change
	UpdateDataFiles		
	
	'Inform user update successful
	Msgbox("Updated " & SelectedGamesMachine.MachineName,"Record Updated")
			
End Sub

Sub DeleteAllChildrenGames(ParentID As Int)
	'this sub will delete all the children game records that have the passed in Parent ID
	
	'removing items will change the size of the List so need a while loop instead
	Dim CurrentListPos As Int
	Dim CurrentGame As Game
	
	CurrentListPos = 0
	
	'keep looping until hit the end of the list, presuming that this comparison is checked each time around the loop
	Do While (CurrentListPos < GamesList.Size)
		
		CurrentGame = GamesList.Get(CurrentListPos)
		
		'check if it matches the passed in parent
		If (CurrentGame.GameMachineID = ParentID) Then
			'need to delete this game
			GamesList.RemoveAt(CurrentListPos)
		
			'don't increase current list pos now, as next item will have been moved back into the slot of the one just deleted
		Else
			'record wasn't a child so next time around the loop need to look at the next item onwards
			CurrentListPos = CurrentListPos + 1			
		End If	
	Loop
	

End Sub

Sub DeleteMachineButton_Click
	'this sub will delete the selected machine
	If (SelectedGamesMachine = Null) Then
		Msgbox("Cannot Delete - No Machine is selected, in fact this should never be possible...","Delete Error")
	End If
	
	'confirm user wants to do this
	Dim DeleteMsg As String
	DeleteMsg = "Delete Games Machine: " & SelectedGamesMachine.MachineName & "? NOTE: This will delete all children records as well"
	
	If (Msgbox2(DeleteMsg,"Delete Machine", "Ok","Cancel","",Null) <> DialogResponse.POSITIVE) Then
		Return 'just leave the subroutine here
	End If 
	
	'first remove the children records
	DeleteAllChildrenGames(SelectedGamesMachine.GameMachineID)
	
	'remove the GamesMachine from the list
	GamesMachinesList.RemoveAt( GamesMachinesList.IndexOf(SelectedGamesMachine) )
	
	'update the on screen controls
	PopulateMachinesListView(True) 
	GameMachineIDEditText.Text = ""
	GameMachineNameEditText.Text = ""
	GameMachineMakerEditText.Text = ""
	GameMachineCostEditText.Text = ""
	GameMachineMostPowerfulCheckBox.Checked = False
	
	'change the record options available for machines
	UpdateMachineButton.Enabled = False
	DeleteMachineButton.Enabled = False
	
	'change the record options available for games and clear its controls
	GameNameEditText.Text = ""
	GameDevEditText.Text = ""
	GameMultiplatformCheckBox.Checked = False
	AddGameButton.Enabled = False
	UpdateGameButton.Enabled = False 
	DeleteGameButton.Enabled = False 
	
	'have no items selected
	SelectedGamesMachine = Null
	SelectedGame = Null
	
	'update the Files
	UpdateDataFiles

	
End Sub

Sub AddMachineButton_Click
	'this sub will add a new game's machine to the list using what is in the edit boxes currently
	Dim NewGamesMachine As GameMachine
	
	NewGamesMachine.Initialize
	NewGamesMachine.MachineCost = GameMachineCostEditText.Text
	NewGamesMachine.MachineMaker = GameMachineMakerEditText.Text
	NewGamesMachine.MachineName = GameMachineNameEditText.Text
	NewGamesMachine.MostPowerful = GameMachineMostPowerfulCheckBox.Checked
	NewGamesMachine.GameMachineID = GetNewGameMachineID 'this calls a function to get the next key
	
	'add the new object to the list
	GamesMachinesList.Add(NewGamesMachine)
		
	'have the on screen controls updated
	PopulateMachinesListView(True)

	'update the global reference to the selected item
	SelectedGamesMachine = NewGamesMachine
	
	'enable the various record options now an item is selected
	UpdateMachineButton.Enabled = True
	DeleteMachineButton.Enabled = True	
	
	'have the data files reflect the change
	UpdateDataFiles
	
End Sub


Sub CopyDefaultFilesButton_Click
	'This sub exists just for debug purposes. It merely copies the default files out of assets into the default ext, regardless if there already exists files there	
	File.Copy(File.DirAssets,"GameMachines.csv",File.DirDefaultExternal,"GameMachines.csv")	
	File.Copy(File.DirAssets,"Games.csv",File.DirDefaultExternal,"Games.csv")
End Sub

Sub UpdateGameButton_Click
	If (SelectedGame = Null) Then
		Msgbox("Cannot update - No Game is selected, in fact this should never be possible...","Update Error")
	End If
	
	'update the data in the object
	SelectedGame.GameName = GameNameEditText.Text
	SelectedGame.GameDev = GameDevEditText.Text
	SelectedGame.PlatformExclusive = GameMultiplatformCheckBox.Checked
	SelectedGame.EGRating = GameEGRatingText.Text
	
	'have the on screen controls updated
	'NOTE: as this is an update am assuming that SelectedGamesMachine still points at the same parent record
	PopulateGamesListView(SelectedGamesMachine.GameMachineID)
		
	'have the data files reflect the change
	UpdateDataFiles		
	
	'Inform user update successful
	Msgbox("Updated " & SelectedGame.GameName,"Record Updated")
	
End Sub

Sub DeleteGameButton_Click
	'this will delete the selected game
	If (SelectedGame = Null) Then
		Msgbox("Cannot Delete - No Game is selected, in fact this should never be possible...","Delete Error")
	End If
	
	'confirm user wants to do this
	Dim DeleteMsg As String
	DeleteMsg = "Delete Game: " & SelectedGame.GameName & "?"
	
	If (Msgbox2(DeleteMsg,"Delete Game", "Ok","Cancel","",Null) <> DialogResponse.POSITIVE) Then
		Return 'just leave the subroutine here
	End If 

	'remove the game from the list
	GamesList.RemoveAt( GamesList.IndexOf(SelectedGame) )
	
	
	'change the record options available for games and clear its controls
	GameNameEditText.Text = ""
	GameDevEditText.Text = ""
	GameMultiplatformCheckBox.Checked = False
	GameEGRatingText.Text = ""
	AddGameButton.Enabled = True
	UpdateGameButton.Enabled = False 
	DeleteGameButton.Enabled = False 
	
	'have no selected game
	SelectedGame = Null
	
	'refresh on screen control
	PopulateGamesListView(SelectedGamesMachine.GameMachineID)	
	
	'update the Files
	UpdateDataFiles
	
End Sub

Sub AddGameButton_Click
	'add a game to the currently selected machine
	'NOTE: this event should only be possible to occur (via button enabling) if a machine is selected
	Dim NewGame As Game
	
	'get data for game from controls, functions and global variable
	NewGame.GameName = GameNameEditText.Text
	NewGame.GameDev = GameDevEditText.Text
	NewGame.PlatformExclusive = GameMultiplatformCheckBox.Checked
	NewGame.EGRating = GameEGRatingText.Text
	NewGame.GameMachineID = SelectedGamesMachine.GameMachineID
	NewGame.GameID = GetNewGameID
	
	'now add the item into the list
	GamesList.Add(NewGame)
	
	'make the selected game be this new game
	SelectedGame = NewGame
	
	'enable the record operation buttons
	UpdateGameButton.Enabled = True 
	DeleteGameButton.Enabled = True 
	
	'have the on screen controls updated
	'NOTE: as this is an update am assuming that SelectedGamesMachine still points at the same parent record
	PopulateGamesListView(SelectedGamesMachine.GameMachineID)
		
	'have the data files reflect the change
	UpdateDataFiles		
	
End Sub

Sub ManualCSVButton_Click
	
	ManualGameMachinesCSVImport
	
End Sub

Sub ManualGameMachinesCSVImport

	Dim FileImportBuffer As List
	
	Dim Delimiter As String
	Dim CurrentImportFileLine As String
	Dim CompleteString As Boolean
	Dim SF As StringFunctions
	Dim CurrentCharacter As String
	
	FileImportBuffer = File.ReadList(File.DirAssets,"GameMachines.csv")
	
	GamesMachinesList.Clear

	For i=0 To FileImportBuffer.Size-1
		
		Dim ImportedMachineData As List		
		Dim CurrentDataString As String
		
		ImportedMachineData.Initialize
		
		CurrentImportFileLine = FileImportBuffer.Get(i)
		
		For n=1 To SF.Len(CurrentImportFileLine)
			CurrentCharacter = SF.Mid(CurrentImportFileLine,n,1)
			
			If CurrentCharacter = "," Then
				ImportedMachineData.Add(CurrentDataString)
				CurrentDataString = ""			
			Else
				CurrentDataString = CurrentDataString & CurrentCharacter	
			End If	
		Next		
		ImportedMachineData.Add(CurrentDataString)
		CurrentDataString = ""	
		
		Dim ImportedGameMachine As GameMachine
		ImportedGameMachine.GameMachineID = ImportedMachineData.Get(0)
		ImportedGameMachine.MachineCost = ImportedMachineData.Get(3)
		ImportedGameMachine.MachineMaker = ImportedMachineData.Get(2)
		ImportedGameMachine.MachineName = ImportedMachineData.Get(1)
		
		If (ImportedMachineData.Get(4) = "TRUE") Then
			ImportedGameMachine.MostPowerful = True
		Else
			ImportedGameMachine.MostPowerful = False
		End If
		
		GamesMachinesList.Add(ImportedGameMachine)				
	Next	
	
	PopulateMachinesListView(True)

End Sub


Sub FullyManualGameMachinesCSVImport
	
	Dim FileImportBuffer As String
	Dim ImportedMachineData As List	
	Dim SF As StringFunctions
	Dim CurrentDataString As String
	Dim CurrentCharacter As String
	Dim FinalDataItem As String
	
	'read in the file's contents
	FileImportBuffer = File.ReadString(File.DirAssets,"GameMachines.csv")
	
	'empty main data storage
	GamesMachinesList.Clear
	
	'prepare individual record extraction list
	ImportedMachineData.Initialize
	
	'loop through all characters in the line
	For n=1 To SF.Len(FileImportBuffer)
		'get the current character
		CurrentCharacter = SF.Mid(FileImportBuffer,n,1)
		
		'check if at the end of a line
		If (CurrentCharacter = CRLF) Then
			'hit the end of a line so need to add the data as a games machine
			
			'first need to add the final piece of data for this line
			'the way B4A interprets Carriage returns means need to trim a final 
			'leading space off of the data before adding it
			FinalDataItem = SF.Trim(CurrentDataString)			
			
			ImportedMachineData.Add(FinalDataItem)
			CurrentDataString = ""
			
			
			'have now extracted a full Game Machine record so can save it to the list
			Dim ImportedGameMachine As GameMachine
			ImportedGameMachine.GameMachineID = ImportedMachineData.Get(0)
			ImportedGameMachine.MachineCost = ImportedMachineData.Get(3)
			ImportedGameMachine.MachineMaker = ImportedMachineData.Get(2)
			ImportedGameMachine.MachineName = ImportedMachineData.Get(1)
			
			If (ImportedMachineData.Get(4) = "TRUE") Then
				ImportedGameMachine.MostPowerful = True
			Else
				ImportedGameMachine.MostPowerful = False
			End If
			
			'add new item to storage
			GamesMachinesList.Add(ImportedGameMachine)
			
			'clear the extracted data storage for the next record
			ImportedMachineData.Clear
		
		Else
			'if the character is a , then 
			If (CurrentCharacter = ",") Then
				'hit a delimiter so save the string and reset the current storage
				ImportedMachineData.Add(CurrentDataString)
				CurrentDataString = ""			
			Else
				'this is data, so keep building it up
				CurrentDataString = CurrentDataString & CurrentCharacter	
			End If			
		End If
	Next				
	
	'see the results in the control
	PopulateMachinesListView(True)	
End Sub

Sub FullyManualImportBtn_Click
	ManualGameMachinesCSVImport
End Sub

Sub ManualCSVExportButton_Click
	'This sub will manually export the GAMES List to a CSV file 
	Dim WriteString As String
	Dim ExportBuffer As List
	ExportBuffer.Initialize
	
	Dim CurrentGame As Game
		
	For i=0 To GamesList.Size-1
		CurrentGame = GamesList.Get(i)						
		
		'IMPORTANT NOTE: THE BELOW LINES WILL NEED TO BE DIFFERENT DEPENDING ON THE DATA STRUCTURE & FILE STRUCTURE OF THE DATA BEING EXPORTED		
		WriteString = CurrentGame.GameID & "," & CurrentGame.GameMachineID & "," & CurrentGame.GameName & "," & CurrentGame.GameDev	
		If (CurrentGame.PlatformExclusive) Then
			WriteString = WriteString & ",TRUE"
		Else
			WriteString = WriteString & ",FALSE"
		End If
		'END OF SECTION THAT MUST BE ALTERED DEPENDING ON DATA STRUCTURE		
		
		ExportBuffer.Add(WriteString)	
	Next 
	
	File.WriteList(File.DirDefaultExternal,"ManualCSVExport_Example.csv",ExportBuffer)
	
	Msgbox("You copied & pasted this code without thinking!","If you see this Msg...")
	
End Sub

Sub PopulateControlSpinners
	'This sub sets up the Sort and Filter with static data because know that extra fields cannot be added apart from in code
	GamesSortSpinner.Clear
	GamesSortSpinner.Add("Default")
	GamesSortSpinner.Add("Game Name")
	GamesSortSpinner.Add("Game Dev")
	GamesSortSpinner.Add("Platform Exclusive")
	GamesSortSpinner.Add("EG Rating")
	GamesSortSpinner.SelectedIndex = 0
	
	
	GamesFilterSpinner.Clear
	GamesFilterSpinner.Add("No Filter")
	GamesFilterSpinner.Add("Game Name")
	GamesFilterSpinner.Add("Game Dev")
	GamesFilterSpinner.Add("Platform Exclusive")	
	GamesFilterSpinner.Add("EG Rating")
	GamesFilterSpinner.SelectedIndex = 0
	
	GameFilterOperatorSpinner.Clear
	GameFilterOperatorSpinner.Add("=")
	GameFilterOperatorSpinner.Add("<")
	GameFilterOperatorSpinner.Add(">")

End Sub

Sub GamesSortSpinner_ItemClick (Position As Int, Value As Object)
	'Selection of Sort item could mean sort has changed so repopulate list
	PopulateGamesListView(SelectedGamesMachine.GameMachineID)
End Sub

Sub GamesFilterSpinner_ItemClick (Position As Int, Value As Object)
	'select of a filter item could mean the filter has changed so repopulate list
	PopulateGamesListView(SelectedGamesMachine.GameMachineID)
End Sub
Sub GameFilterOperatorSpinner_ItemClick (Position As Int, Value As Object)
	'select of a filter operator could mean the filter has changed so repopulate list
	PopulateGamesListView(SelectedGamesMachine.GameMachineID)
End Sub